@@ -25,33 +25,49 @@
 #include <QSet>
 #include <QMap>
 
-#include <KDEDModule>
-#include <KPluginFactory>
 #include "kdeconnectcore_export.h"
+#include "device.h"
 
-class DaemonPrivate;
 class NetworkPackage;
 class DeviceLink;
+class Device;
+class QNetworkAccessManager;
 
 class KDECONNECTCORE_EXPORT Daemon
     : public QObject
 {
     Q_OBJECT
     Q_CLASSINFO("D-Bus Interface", "org.kde.kdeconnect.daemon")
+    Q_PROPERTY(bool isDiscoveringDevices READ isDiscoveringDevices)
 
 public:
-    Daemon(QObject *parent);
-    ~Daemon();
+    explicit Daemon(QObject *parent, bool testMode = false);
+    ~Daemon() override;
+
+    static Daemon* instance();
+
+    QList<Device*> devicesList() const;
+
+    virtual void askPairingConfirmation(PairingHandler *d) = 0;
+    virtual void reportError(const QString &title, const QString &description) = 0;
+    virtual QNetworkAccessManager* networkAccessManager();
+
+    Device* getDevice(const QString& deviceId);
 
 public Q_SLOTS:
-
-    //After calling this, signal deviceDiscovered will be triggered for each device
-    Q_SCRIPTABLE void setDiscoveryEnabled(bool b);
+    Q_SCRIPTABLE void acquireDiscoveryMode(const QString &id);
+    Q_SCRIPTABLE void releaseDiscoveryMode(const QString &id);
 
     Q_SCRIPTABLE void forceOnNetworkChange();
 
+    ///don't try to turn into Q_PROPERTY, it doesn't work
+    Q_SCRIPTABLE QString announcedName();
+    Q_SCRIPTABLE void setAnnouncedName(const QString &name);
+
     //Returns a list of ids. The respective devices can be manipulated using the dbus path: "/modules/kdeconnect/Devices/"+id
-    Q_SCRIPTABLE QStringList devices(bool onlyReachable = false, bool onlyVisible = false) const;
+    Q_SCRIPTABLE QStringList devices(bool onlyReachable = false, bool onlyPaired = false) const;
+
+    Q_SCRIPTABLE QString deviceIdByName(const QString &name) const;
 
 Q_SIGNALS:
     Q_SCRIPTABLE void deviceAdded(const QString& id);
@@ -60,10 +76,14 @@ Q_SIGNALS:
 
 private Q_SLOTS:
     void onNewDeviceLink(const NetworkPackage& identityPackage, DeviceLink* dl);
-    void onDeviceReachableStatusChanged();
+    void onDeviceStatusChanged();
 
 private:
-    QScopedPointer<DaemonPrivate> d;
+    bool isDiscoveringDevices() const;
+    void removeDevice(Device* d);
+    void cleanDevices();
+
+    QScopedPointer<struct DaemonPrivate> d;
 };
 
 #endif
