@@ -23,12 +23,11 @@
 
 #include <QObject>
 #include <QString>
-#include <QMap>
-#include <QSslKey>
-#include <QTimer>
-#include <QtCrypto>
+#include <QVector>
+#include <QSet>
 
 #include "networkpackage.h"
+#include "backends/devicelink.h"
 
 class DeviceLink;
 class KdeConnectPlugin;
@@ -38,16 +37,15 @@ class KDECONNECTCORE_EXPORT Device
 {
     Q_OBJECT
     Q_CLASSINFO("D-Bus Interface", "org.kde.kdeconnect.device")
-    Q_PROPERTY(QString id READ id CONSTANT)
+    Q_PROPERTY(QString type READ type CONSTANT)
+    Q_PROPERTY(QString name READ name NOTIFY nameChanged)
     Q_PROPERTY(QString iconName READ iconName CONSTANT)
-    Q_PROPERTY(QString name READ name)
+    Q_PROPERTY(QString statusIconName READ statusIconName)
+    Q_PROPERTY(bool isReachable READ isReachable NOTIFY reachableStatusChanged)
+    Q_PROPERTY(bool isTrusted READ isTrusted NOTIFY trustedChanged)
+    Q_PROPERTY(QStringList supportedPlugins READ supportedPlugins NOTIFY pluginsChanged)
 
-    enum PairStatus {
-        NotPaired,
-        Requested,
-        RequestedByPeer,
-        Paired,
-    };
+public:
 
     enum DeviceType {
         Unknown,
@@ -56,12 +54,14 @@ class KDECONNECTCORE_EXPORT Device
         Phone,
         Tablet,
     };
-    static DeviceType str2type(QString deviceType);
-    static QString type2str(DeviceType deviceType);
 
-public:
+    enum TrustStatus {
+        NotTrusted,
+        Trusted
+    };
+
     /**
-     * Reads the @p device from KConfig
+     * Restores the @p device from the saved configuration
      *
      * We already know it but we need to wait for an incoming DeviceLink to communicate
      */
@@ -74,75 +74,81 @@ public:
      */
     Device(QObject* parent, const NetworkPackage& np, DeviceLink* dl);
 
-    virtual ~Device();
+    ~Device() override;
 
     QString id() const { return m_deviceId; }
     QString name() const { return m_deviceName; }
     QString dbusPath() const { return "/modules/kdeconnect/devices/"+id(); }
+    QString type() const { return type2str(m_deviceType); }
     QString iconName() const;
+    QString statusIconName() const;
+    Q_SCRIPTABLE QString encryptionInfo() const;
 
     //Add and remove links
     void addLink(const NetworkPackage& identityPackage, DeviceLink*);
     void removeLink(DeviceLink*);
 
-    QString privateKeyPath() const;
-    
-    Q_SCRIPTABLE bool isPaired() const { return m_pairStatus==Device::Paired; }
-    Q_SCRIPTABLE bool pairRequested() const { return m_pairStatus==Device::Requested; }
+    Q_SCRIPTABLE bool isTrusted() const;
 
     Q_SCRIPTABLE QStringList availableLinks() const;
-    Q_SCRIPTABLE bool isReachable() const { return !m_deviceLinks.isEmpty(); }
+    bool isReachable() const { return !m_deviceLinks.isEmpty(); }
 
     Q_SCRIPTABLE QStringList loadedPlugins() const;
     Q_SCRIPTABLE bool hasPlugin(const QString& name) const;
 
+    Q_SCRIPTABLE QString pluginsConfigFile() const;
+
+    KdeConnectPlugin* plugin(const QString& pluginName) const;
+    void setPluginEnabled(const QString& pluginName, bool enabled);
+    bool isPluginEnabled(const QString& pluginName) const;
+
+    void cleanUnneededLinks();
+
+    int protocolVersion() { return m_protocolVersion; }
+    QStringList supportedPlugins() const { return m_supportedPlugins.toList(); }
+
 public Q_SLOTS:
     ///sends a @p np package to the device
+    ///virtual for testing purposes.
     virtual bool sendPackage(NetworkPackage& np);
 
     //Dbus operations
 public Q_SLOTS:
-    Q_SCRIPTABLE void requestPair();
-    Q_SCRIPTABLE void unpair();
-    Q_SCRIPTABLE void reloadPlugins(); //From kconf
-    void acceptPairing();
-    void rejectPairing();
+    Q_SCRIPTABLE void requestPair(); //to all links
+    Q_SCRIPTABLE void unpair(); //from all links
+    Q_SCRIPTABLE void reloadPlugins(); //from kconf
 
 private Q_SLOTS:
     void privateReceivedPackage(const NetworkPackage& np);
     void linkDestroyed(QObject* o);
-    void pairingTimeout();
+    void pairStatusChanged(DeviceLink::PairStatus current);
 
 Q_SIGNALS:
-    Q_SCRIPTABLE void reachableStatusChanged();
     Q_SCRIPTABLE void pluginsChanged();
-    Q_SCRIPTABLE void pairingSuccesful();
-    Q_SCRIPTABLE void pairingFailed(const QString& error);
-    Q_SCRIPTABLE void unpaired();
+    Q_SCRIPTABLE void reachableStatusChanged();
+    Q_SCRIPTABLE void trustedChanged(bool trusted);
+    Q_SCRIPTABLE void pairingError(const QString& error);
+    Q_SCRIPTABLE void nameChanged(const QString& name);
 
-private:
+private: //Methods
+    static DeviceType str2type(const QString &deviceType);
+    static QString type2str(DeviceType deviceType);
+
+    void setName(const QString &name);
+    QString iconForStatus(bool reachable, bool paired) const;
+
+private: //Fields (TODO: dPointer!)
     const QString m_deviceId;
     QString m_deviceName;
     DeviceType m_deviceType;
-    QCA::PrivateKey m_privateKey;
-    QCA::PublicKey m_publicKey;
-    PairStatus m_pairStatus;
     int m_protocolVersion;
 
-    QList<DeviceLink*> m_deviceLinks;
-    QMap<QString, KdeConnectPlugin*> m_plugins;
-    QMultiMap<QString, KdeConnectPlugin*> m_pluginsByIncomingInterface;
-    QMultiMap<QString, KdeConnectPlugin*> m_pluginsByOutgoingInterface;
-
-    QTimer m_pairingTimeut;
-    QSet<QString> m_incomingCapabilities;
-    QSet<QString> m_outgoingCapabilities;
-
-    void setAsPaired();
-    void storeAsTrusted();
-    bool sendOwnPublicKey();
-    void initPrivateKey();
+    QVector<DeviceLink*> m_deviceLinks;
+    QHash<QString, KdeConnectPlugin*> m_plugins;
 
+    //Capabilities stuff
+    QMultiMap<QString, KdeConnectPlugin*> m_pluginsByIncomingCapability;
+    QSet<QString> m_supportedPlugins;
 };
 
 Q_DECLARE_METATYPE(Device*)
