@@ -22,12 +22,10 @@
 #define PLUGINLOADER_H
 
 #include <QObject>
-#include <QMap>
+#include <QHash>
 #include <QString>
 
-#include <KPluginFactory>
-#include <KService>
-#include <KPluginInfo>
+#include <KPluginMetaData>
 
 class Device;
 class KdeConnectPlugin;
@@ -38,16 +36,17 @@ class PluginLoader
 public:
     static PluginLoader* instance();
 
-    QStringList incomingInterfaces() const;
-    QStringList outgoingInterfaces() const;
     QStringList getPluginList() const;
-    KPluginInfo getPluginInfo(const QString& name) const;
-    KService::Ptr pluginService(const QString& pluginName) const;
+    KPluginMetaData getPluginInfo(const QString& name) const;
     KdeConnectPlugin* instantiatePluginForDevice(const QString& name, Device* device) const;
 
+    QStringList incomingCapabilities() const;
+    QStringList outgoingCapabilities() const;
+    QSet<QString> pluginsForCapabilities(const QSet<QString> &incoming, const QSet<QString> &outgoing);
+
 private:
     PluginLoader();
-    QMap<QString,KService::Ptr> plugins;
+    QHash<QString, KPluginMetaData> plugins;
 
 
 };
