@@ -21,19 +21,12 @@
 #ifndef KDECONNECTKCM_H
 #define KDECONNECTKCM_H
 
+#include <KCModule>
 #include <QStandardItemModel>
-#include <QDBusConnection>
-
-#include <kcmodule.h>
-#include <ksharedconfig.h>
 
 class QModelIndex;
-class AccountsModel;
-class AccountWidget;
-class QStackedLayout;
-class QItemSelectionModel;
-class QDBusInterface;
 class DeviceDbusInterface;
+class DaemonDbusInterface;
 class DevicesModel;
 class DevicesSortProxyModel;
 
@@ -47,10 +40,12 @@ class KdeConnectKcm
     Q_OBJECT
 public:
     KdeConnectKcm(QWidget *parent, const QVariantList&);
-    virtual ~KdeConnectKcm();
+    ~KdeConnectKcm() override;
 
 private:
-    virtual void save();
+    void save() override;
+    QSize sizeHint() const override;
+    QSize minimumSizeHint() const override;
 
 private Q_SLOTS:
     void deviceSelected(const QModelIndex& current);
@@ -58,17 +53,24 @@ private Q_SLOTS:
     void pluginsConfigChanged();
     void sendPing();
     void resetSelection();
-    void pairingSuccesful();
+    void trustedChanged(bool);
     void pairingFailed(const QString& error);
-    void unpaired();
     void refresh();
+    void renameShow();
+    void renameDone();
+    void setRenameMode(bool b);
+    void resetCurrentDevice();
 
 private:
+    void resetDeviceView();
+
     Ui::KdeConnectKcmUi* kcmUi;
+    DaemonDbusInterface* daemon;
     DevicesModel* devicesModel;
     DevicesSortProxyModel* sortProxyModel;
     DeviceDbusInterface* currentDevice;
     QModelIndex currentIndex;
+    QStringList m_oldSupportedPluginNames;
 
 public Q_SLOTS:
     void unpair();
