@@ -21,14 +21,15 @@
 #include "batteryplugin.h"
 
 #include <KNotification>
-#include <KIcon>
+#include <QIcon>
 #include <KLocalizedString>
+#include <KPluginFactory>
 
-#include <core/kdebugnamespace.h>
 #include "batterydbusinterface.h"
 
-K_PLUGIN_FACTORY( KdeConnectPluginFactory, registerPlugin< BatteryPlugin >(); )
-K_EXPORT_PLUGIN( KdeConnectPluginFactory("kdeconnect_battery", "kdeconnect-plugins") )
+K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_battery.json", registerPlugin< BatteryPlugin >(); )
+
+Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_BATTERY, "kdeconnect.plugin.battery")
 
 BatteryPlugin::BatteryPlugin(QObject *parent, const QVariantList &args)
     : KdeConnectPlugin(parent, args)
@@ -42,8 +43,7 @@ BatteryPlugin::BatteryPlugin(QObject *parent, const QVariantList &args)
 
 void BatteryPlugin::connected()
 {
-    NetworkPackage np(PACKAGE_TYPE_BATTERY);
-    np.set("request",true);
+    NetworkPackage np(PACKAGE_TYPE_BATTERY_REQUEST, {{"request",true}});
     sendPackage(np);
 }
 
@@ -60,8 +60,8 @@ BatteryPlugin::~BatteryPlugin()
 
 bool BatteryPlugin::receivePackage(const NetworkPackage& np)
 {
-    bool isCharging = np.get<bool>("isCharging");
-    int currentCharge = np.get<int>("currentCharge");
+    bool isCharging = np.get<bool>("isCharging", false);
+    int currentCharge = np.get<int>("currentCharge", -1);
     int thresholdEvent = np.get<int>("thresholdEvent", (int)ThresholdNone);
 
     if (batteryDbusInterface->charge() != currentCharge
@@ -72,9 +72,9 @@ bool BatteryPlugin::receivePackage(const NetworkPackage& np)
 
     if ( thresholdEvent == ThresholdBatteryLow && !isCharging ) {
         KNotification* notification = new KNotification("batteryLow");
-        notification->setPixmap(KIcon("battery-040").pixmap(48, 48));
-        notification->setComponentData(KComponentData("kdeconnect", "kdeconnect-kded"));
-        notification->setTitle(i18nc("device name: low battery", "%1: low battery", device()->name()));
+        notification->setIconName("battery-040");
+        notification->setComponentName("kdeconnect");
+        notification->setTitle(i18nc("device name: low battery", "%1: Low Battery", device()->name()));
         notification->setText(i18n("Battery at %1%", currentCharge));
         notification->sendEvent();
     }
@@ -83,3 +83,4 @@ bool BatteryPlugin::receivePackage(const NetworkPackage& np)
 
 }
 
+#include "batteryplugin.moc"
