@@ -26,82 +26,93 @@
 #include <qdbusconnectioninterface.h>
 #include <QDBusReply>
 #include <QDBusMessage>
+#include <QDBusServiceWatcher>
+
+#include <QImage>
+#include <QBuffer>
+#include <QByteArray>
+
+#include <KPluginFactory>
 
-#include <core/kdebugnamespace.h>
 #include <core/device.h>
 #include "mprisdbusinterface.h"
 #include "propertiesdbusinterface.h"
 
-K_PLUGIN_FACTORY( KdeConnectPluginFactory, registerPlugin< MprisControlPlugin >(); )
-K_EXPORT_PLUGIN( KdeConnectPluginFactory("kdeconnect_mpriscontrol", "kdeconnect-plugins") )
+K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_mpriscontrol.json", registerPlugin< MprisControlPlugin >(); )
+
+Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_MPRIS, "kdeconnect.plugin.mpris")
 
 MprisControlPlugin::MprisControlPlugin(QObject* parent, const QVariantList& args)
     : KdeConnectPlugin(parent, args)
+    , prevVolume(-1)
 {
+    m_watcher = new QDBusServiceWatcher(QString(), QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this);
 
-    //Detect new interfaces
-    connect(QDBusConnection::sessionBus().interface(), SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-            this, SLOT(serviceOwnerChanged(QString,QString,QString)));
+    // TODO: QDBusConnectionInterface::serviceOwnerChanged is deprecated, maybe query org.freedesktop.DBus directly?
+    connect(QDBusConnection::sessionBus().interface(), &QDBusConnectionInterface::serviceOwnerChanged, this, &MprisControlPlugin::serviceOwnerChanged);
 
     //Add existing interfaces
-    QStringList interfaces = QDBusConnection::sessionBus().interface()->registeredServiceNames().value();
-    Q_FOREACH (const QString& iface, interfaces) {
-        if (iface.startsWith("org.mpris.MediaPlayer2")) {
-            addPlayer(iface);
-        }
+    QStringList services = QDBusConnection::sessionBus().interface()->registeredServiceNames().value();
+    Q_FOREACH (const QString& service, services) {
+        // The string doesn't matter, it just needs to be empty/non-empty
+        serviceOwnerChanged(service, QLatin1String(""), QLatin1String("1"));
     }
-
 }
 
-void MprisControlPlugin::serviceOwnerChanged(const QString &name,
-                                              const QString &oldOwner,
-                                              const QString &newOwner)
+// Copied from the mpris2 dataengine in the plasma-workspace repository
+void MprisControlPlugin::serviceOwnerChanged(const QString& serviceName, const QString& oldOwner, const QString& newOwner)
 {
-    Q_UNUSED(oldOwner);
+    if (!serviceName.startsWith(QLatin1String("org.mpris.MediaPlayer2.")))
+        return;
 
-    if (name.startsWith("org.mpris.MediaPlayer2")) {
+    if (!oldOwner.isEmpty()) {
+        qCDebug(KDECONNECT_PLUGIN_MPRIS) << "MPRIS service" << serviceName << "just went offline";
+        removePlayer(serviceName);
+    }
 
-        kDebug(debugArea()) << "Mpris (un)registered in bus" << name << oldOwner << newOwner;
-
-        if (oldOwner.isEmpty()) {
-            addPlayer(name);
-        } else if (newOwner.isEmpty()) {
-            removePlayer(name);
-        }
+    if (!newOwner.isEmpty()) {
+        qCDebug(KDECONNECT_PLUGIN_MPRIS) << "MPRIS service" << serviceName << "just came online";
+        addPlayer(serviceName);
     }
 }
 
+
 void MprisControlPlugin::addPlayer(const QString& service)
 {
     QDBusInterface mprisInterface(service, "/org/mpris/MediaPlayer2", "org.mpris.MediaPlayer2");
     //FIXME: This call hangs and returns an empty string if KDED is still starting!
-    const QString identity = mprisInterface.property("Identity").toString();
+    QString identity = mprisInterface.property("Identity").toString();
+    if (identity.isEmpty()) {
+        identity = service.mid(sizeof("org.mpris.MediaPlayer2"));
+    }
     playerList[identity] = service;
-    kDebug(debugArea()) << "Mpris addPlayer" << service << "->" << identity;
+    qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Mpris addPlayer" << service << "->" << identity;
     sendPlayerList();
 
     OrgFreedesktopDBusPropertiesInterface* freedesktopInterface = new OrgFreedesktopDBusPropertiesInterface(service, "/org/mpris/MediaPlayer2", QDBusConnection::sessionBus(), this);
-    connect(freedesktopInterface, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)), this, SLOT(propertiesChanged(QString, QVariantMap)));
+    connect(freedesktopInterface, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)), this, SLOT(propertiesChanged(QString,QVariantMap)));
 
     OrgMprisMediaPlayer2PlayerInterface* mprisInterface0  = new OrgMprisMediaPlayer2PlayerInterface(service, "/org/mpris/MediaPlayer2", QDBusConnection::sessionBus());
     connect(mprisInterface0, SIGNAL(Seeked(qlonglong)), this, SLOT(seeked(qlonglong)));
 }
 
 void MprisControlPlugin::seeked(qlonglong position){
-    kDebug(debugArea()) << "Seeked in player";
-    NetworkPackage np(PACKAGE_TYPE_MPRIS);
-    np.set("pos", position/1000); //Send milis instead of nanos
+    //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Seeked in player";
     OrgFreedesktopDBusPropertiesInterface* interface = (OrgFreedesktopDBusPropertiesInterface*)sender();
     const QString& service = interface->service();
     const QString& player = playerList.key(service);
-    np.set("player", player);
+
+    NetworkPackage np(PACKAGE_TYPE_MPRIS, {
+        {"pos", position/1000}, //Send milis instead of nanos
+        {"player", player}
+    });
     sendPackage(np);
 }
 
 void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, const QVariantMap& properties)
 {
     Q_UNUSED(propertyInterface);
-    
+
     NetworkPackage np(PACKAGE_TYPE_MPRIS);
     bool somethingToSend = false;
     if (properties.contains("Volume")) {
@@ -131,6 +142,32 @@ void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, con
             }
             somethingToSend = true;
         }
+        if (nowPlayingMap.contains("mpris:artUrl")) {
+            const QUrl artUrl(nowPlayingMap["mpris:artUrl"].toString());
+
+            /*
+             * We only handle images stored locally right now but it should be easy
+             * enough to download remote images and send them.
+             */
+            const int artMaxWidth = 512;
+            const int artMaxHeight = 512;
+            if (artUrl.isLocalFile()) {
+                QImage artImage(artUrl.path());
+
+                // Scale the image to a sane size
+                artImage = artImage.scaled({artMaxWidth, artMaxHeight}, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+
+                QByteArray imageBufferArray;
+                QBuffer imageBuffer(&imageBufferArray);
+                imageBuffer.open(QIODevice::WriteOnly);
+                artImage.save(&imageBuffer, "PNG");
+
+                const QString base64Image = QString::fromLatin1(imageBufferArray.toBase64());
+                np.set("artImage", base64Image);
+                somethingToSend = true;
+            }
+
+        }
 
     }
     if (properties.contains("PlaybackStatus")) {
@@ -145,7 +182,7 @@ void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, con
         const QString& player = playerList.key(service);
         np.set("player", player);
         // Always also update the position
-        OrgMprisMediaPlayer2PlayerInterface mprisInterface(playerList[player], "/org/mpris/MediaPlayer2", QDBusConnection::sessionBus());
+        OrgMprisMediaPlayer2PlayerInterface mprisInterface(service, "/org/mpris/MediaPlayer2", QDBusConnection::sessionBus());
         if (mprisInterface.canSeek()) {
             long long pos = mprisInterface.position();
             np.set("pos", pos/1000); //Send milis instead of nanos
@@ -157,7 +194,7 @@ void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, con
 void MprisControlPlugin::removePlayer(const QString& ifaceName)
 {
     const QString identity = playerList.key(ifaceName);
-    kDebug(debugArea()) << "Mpris removePlayer" << ifaceName << "->" << identity;
+    qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Mpris removePlayer" << ifaceName << "->" << identity;
     playerList.remove(identity);
     sendPlayerList();
 }
@@ -180,27 +217,28 @@ bool MprisControlPlugin::receivePackage (const NetworkPackage& np)
 
     //Do something to the mpris interface
     OrgMprisMediaPlayer2PlayerInterface mprisInterface(playerList[player], "/org/mpris/MediaPlayer2", QDBusConnection::sessionBus());
+    mprisInterface.setTimeout(500);
     if (np.has("action")) {
         const QString& action = np.get<QString>("action");
-        kDebug(debugArea()) << "Calling action" << action << "in" << playerList[player];
+        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Calling action" << action << "in" << playerList[player];
         //TODO: Check for valid actions, currently we trust anything the other end sends us
         mprisInterface.call(action);
     }
     if (np.has("setVolume")) {
         double volume = np.get<int>("setVolume")/100.f;
-        kDebug(debugArea()) << "Setting volume" << volume << "to" << playerList[player];
+        qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Setting volume" << volume << "to" << playerList[player];
         mprisInterface.setVolume(volume);
     }
     if (np.has("Seek")) {
         int offset = np.get<int>("Seek");
-        kDebug(debugArea()) << "Seeking" << offset << "to" << playerList[player];
+        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Seeking" << offset << "to" << playerList[player];
         mprisInterface.Seek(offset);
     }
 
     if (np.has("SetPosition")){
         qlonglong position = np.get<qlonglong>("SetPosition",0)*1000;
         qlonglong seek = position - mprisInterface.position();
-        kDebug(debugArea()) << "Setting position by seeking" << seek << "to" << playerList[player];
+        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Setting position by seeking" << seek << "to" << playerList[player];
         mprisInterface.Seek(seek);
     }
 
@@ -213,15 +251,16 @@ bool MprisControlPlugin::receivePackage (const NetworkPackage& np)
         QString nowPlaying = nowPlayingMap["xesam:title"].toString();
         if (nowPlayingMap.contains("xesam:artist")) {
             nowPlaying = nowPlayingMap["xesam:artist"].toString() + " - " + nowPlaying;
-        }if (nowPlayingMap.contains("mpris:length")) {
+        }
+        answer.set("nowPlaying",nowPlaying);
+
+        if (nowPlayingMap.contains("mpris:length")) {
             qlonglong length = nowPlayingMap["mpris:length"].toLongLong();
             answer.set("length",length/1000);
         }
         qlonglong pos = mprisInterface.position();
         answer.set("pos", pos/1000);
 
-        answer.set("nowPlaying",nowPlaying);
-
         bool playing = (mprisInterface.playbackStatus() == QLatin1String("Playing"));
         answer.set("isPlaying", playing);
 
@@ -246,3 +285,5 @@ void MprisControlPlugin::sendPlayerList()
     np.set("playerList",playerList.keys());
     sendPackage(np);
 }
+
+#include "mpriscontrolplugin.moc"
