@@ -24,7 +24,8 @@
 #include <knotification.h>
 
 #include <core/kdeconnectplugin.h>
-#define PACKAGE_TYPE_NOTIFICATION QLatin1String("kdeconnect.notification")
+
+#define PACKAGE_TYPE_NOTIFICATION_REQUEST QLatin1String("kdeconnect.notification.request")
 
 /*
  * This class is just a proxy for NotificationsDbusInterface
@@ -32,6 +33,7 @@
  * KdeConnectPlugin at the same time (both are QObject)
  */
 class NotificationsDbusInterface;
+class NotificationsListener;
 
 class NotificationsPlugin
     : public KdeConnectPlugin
@@ -40,14 +42,15 @@ class NotificationsPlugin
 
 public:
     explicit NotificationsPlugin(QObject *parent, const QVariantList &args);
-    virtual ~NotificationsPlugin();
+    ~NotificationsPlugin() override;
     
 public Q_SLOTS:
-    virtual bool receivePackage(const NetworkPackage& np);
-    virtual void connected();
+    bool receivePackage(const NetworkPackage& np) override;
+    void connected() override;
 
-private:
+protected:
     NotificationsDbusInterface* notificationsDbusInterface;
+    NotificationsListener* notificationsListener;
 
 };
 
