@@ -21,29 +21,27 @@
 #ifndef PAUSEMUSIC_CONFIG_H
 #define PAUSEMUSIC_CONFIG_H
 
-#include <kcmodule.h>
-#include <ksharedconfig.h>
+#include "kcmplugin/kdeconnectpluginkcm.h"
 
 namespace Ui {
     class PauseMusicConfigUi;
 }
 
 class PauseMusicConfig
-    : public KCModule
+    : public KdeConnectPluginKcm
 {
     Q_OBJECT
 public:
     PauseMusicConfig(QWidget *parent, const QVariantList&);
-    virtual ~PauseMusicConfig();
+    ~PauseMusicConfig() override;
 
 public Q_SLOTS:
-    virtual void save();
-    virtual void load();
-    virtual void defaults();
+    void save() override;
+    void load() override;
+    void defaults() override;
 
 private:
     Ui::PauseMusicConfigUi* m_ui;
-    KSharedConfigPtr m_cfg;
 
 };
 
