@@ -23,17 +23,13 @@
 #include <QDBusConnection>
 #include <QDBusInterface>
 #include <QDBusConnectionInterface>
-#include <QDBusReply>
 #include <QDBusMessage>
+#include <QDBusReply>
+#include <QDebug>
 
-#include <KSharedConfig>
-#include <KConfigGroup>
+#include <KPluginFactory>
 
-#include <core/kdebugnamespace.h>
-#include <core/networkpackage.h>
-
-K_PLUGIN_FACTORY( KdeConnectPluginFactory, registerPlugin< PauseMusicPlugin >(); )
-K_EXPORT_PLUGIN( KdeConnectPluginFactory("kdeconnect_pausemusic", "kdeconnect-plugins") )
+K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_pausemusic.json", registerPlugin< PauseMusicPlugin >(); )
 
 //TODO: Port this away from KMix to use only Pulseaudio
 int PauseMusicPlugin::isKMixMuted() {
@@ -56,7 +52,6 @@ int PauseMusicPlugin::isKMixMuted() {
     return (mixerInterface.property("volume").toInt() == 0);
 }
 
-
 PauseMusicPlugin::PauseMusicPlugin(QObject* parent, const QVariantList& args)
     : KdeConnectPlugin(parent, args)
     , muted(false)
@@ -66,9 +61,7 @@ PauseMusicPlugin::PauseMusicPlugin(QObject* parent, const QVariantList& args)
 
 bool PauseMusicPlugin::receivePackage(const NetworkPackage& np)
 {
-    //FIXME: There should be a better way to listen to changes in the config file instead of reading the value each time
-    KSharedConfigPtr config = KSharedConfig::openConfig("kdeconnect/plugins/pausemusic");
-    bool pauseOnlyWhenTalking = config->group("condition").readEntry("talking_only", false);
+    bool pauseOnlyWhenTalking = config()->get("conditionTalking", false);
 
     if (pauseOnlyWhenTalking) {
         if (np.get<QString>("event") != "talking") {
@@ -82,8 +75,8 @@ bool PauseMusicPlugin::receivePackage(const NetworkPackage& np)
 
     bool pauseConditionFulfilled = !np.get<bool>("isCancel");
 
-    bool pause = config->group("actions").readEntry("pause", true);
-    bool mute = config->group("actions").readEntry("mute", false);
+    bool pause = config()->get("actionPause", true);
+    bool mute = config()->get("actionMute", false);
 
     if (pauseConditionFulfilled) {
 
@@ -129,12 +122,7 @@ bool PauseMusicPlugin::receivePackage(const NetworkPackage& np)
         if (pause && !pausedSources.empty()) {
             Q_FOREACH (const QString& iface, pausedSources) {
                 QDBusInterface mprisInterface(iface, "/org/mpris/MediaPlayer2", "org.mpris.MediaPlayer2.Player");
-                //Calling play does not work for Spotify
-                //mprisInterface->call(QDBus::Block,"Play");
-                //Workaround: Using playpause instead (checking first if it is already playing)
-                QString status = mprisInterface.property("PlaybackStatus").toString();
                 mprisInterface.asyncCall("PlayPause");
-                //End of workaround
             }
             pausedSources.clear();
         }
@@ -144,3 +132,5 @@ bool PauseMusicPlugin::receivePackage(const NetworkPackage& np)
     return true;
 
 }
+
+#include "pausemusicplugin.moc"
