@@ -18,28 +18,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QDir>
-#include <QTimerEvent>
-
-#include <KLocalizedString>
-#include <KStandardDirs>
-
-#include <core/kdebugnamespace.h>
-
 #include "mounter.h"
 
-static const char* idleTimeout_c = "idleTimeout";
+#include <QDir>
+#include <QDebug>
 
-Mounter::Mounter(SftpPlugin* sftp, int idleTimeout)
+#include <KLocalizedString>
+
+#include "mountloop.h"
+#include "sftp_debug.h"
+#include "kdeconnectconfig.h"
+
+Mounter::Mounter(SftpPlugin* sftp)
     : QObject(sftp)
     , m_sftp(sftp)
-    , m_proc(0)
-    , m_id(generateId())
-    , m_mpoint(m_sftp->mountPoint())
+    , m_proc(nullptr)
+    , m_mountPoint(sftp->mountPoint())
     , m_started(false)
-{ 
-    Q_ASSERT(sftp);
-  
+{
+
     connect(m_sftp, SIGNAL(packageReceived(NetworkPackage)), this, SLOT(onPakcageReceived(NetworkPackage)));
 
     connect(&m_connectTimer, SIGNAL(timeout()), this, SLOT(onMountTimeout()));
@@ -47,25 +44,18 @@ Mounter::Mounter(SftpPlugin* sftp, int idleTimeout)
     connect(this, SIGNAL(mounted()), &m_connectTimer, SLOT(stop()));
     connect(this, SIGNAL(failed(QString)), &m_connectTimer, SLOT(stop()));
 
-    if (idleTimeout)
-    {
-        connect(&m_idleTimer, SIGNAL(timeout()), this, SLOT(onIdleTimeout()));
-    }
-
     m_connectTimer.setInterval(10000);
     m_connectTimer.setSingleShot(true);
-    
-    m_idleTimer.setInterval(idleTimeout);
-    m_idleTimer.setSingleShot(false);
-    
+
     QTimer::singleShot(0, this, SLOT(start()));
-    kDebug(debugArea()) << "Created";
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Created mounter";
 }
 
 Mounter::~Mounter()
 {
-    unmount();
-    kDebug(debugArea()) << "Destroyed";
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Destroy mounter";
+    //FIXME: We don't unmount becuase it crashes if we try to unmount while the filesystem is being used. Potential memory leak.
+    //unmount();
 }
 
 bool Mounter::wait()
@@ -75,7 +65,7 @@ bool Mounter::wait()
         return true;
     }
     
-    kDebug(debugArea()) << "Starting loop to wait for mount";
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Starting loop to wait for mount";
     
     MountLoop loop;
     connect(this, SIGNAL(mounted()), &loop, SLOT(successed()));
@@ -87,42 +77,40 @@ void Mounter::onPakcageReceived(const NetworkPackage& np)
 {
     if (np.get<bool>("stop", false))
     {
-        kDebug(debugArea()) << "SFTP server stopped";
+        qCDebug(KDECONNECT_PLUGIN_SFTP) << "SFTP server stopped";
         unmount();
         return;
     }
-    
-    //TODO implement on android side
-    //if (np.get<int>("id") != m_id) return;
 
     //This is the previous code, to access sftp server using KIO. Now we are
     //using the external binary sshfs, and accessing it as a local filesystem.
   /*
-   *    KUrl url;
-   *    url.setProtocol("sftp");
+   *    QUrl url;
+   *    url.setScheme("sftp");
    *    url.setHost(np.get<QString>("ip"));
    *    url.setPort(np.get<QString>("port").toInt());
-   *    url.setUser(np.get<QString>("user"));
-   *    url.setPass(np.get<QString>("password"));
+   *    url.setUserName(np.get<QString>("user"));
+   *    url.setPassword(np.get<QString>("password"));
    *    url.setPath(np.get<QString>("path"));
    *    new KRun(url, 0);
    *    Q_EMIT mounted();
    */
 
-    m_proc.reset(new KProcess(this));
+    unmount();
+
+    m_proc = new KProcess(this);
     m_proc->setOutputChannelMode(KProcess::MergedChannels);
 
-    connect(m_proc.data(), SIGNAL(started()), SLOT(onStarted()));    
-    connect(m_proc.data(), SIGNAL(error(QProcess::ProcessError)), SLOT(onError(QProcess::ProcessError)));
-    connect(m_proc.data(), SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(onFinished(int,QProcess::ExitStatus)));
+    connect(m_proc, SIGNAL(started()), SLOT(onStarted()));
+    connect(m_proc, SIGNAL(error(QProcess::ProcessError)), SLOT(onError(QProcess::ProcessError)));
+    connect(m_proc, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(onFinished(int,QProcess::ExitStatus)));
 
-    const QString mpoint = m_sftp->mountPoint();
-    QDir().mkpath(mpoint);
+    QDir().mkpath(m_mountPoint);
 
     const QString program = "sshfs";
 
     QString path;
-    if (np.has("multiPaths")) path = "/";
+    if (np.has("multiPaths")) path = '/';
     else path = np.get<QString>("path");
 
     const QStringList arguments = QStringList()
@@ -130,48 +118,48 @@ void Mounter::onPakcageReceived(const NetworkPackage& np)
             .arg(np.get<QString>("user"))
             .arg(np.get<QString>("ip"))
             .arg(path)
-        << mpoint            
+        << m_mountPoint
         << "-p" << np.get<QString>("port")
-        << "-d"
         << "-f"
-        << "-o" << "IdentityFile=" + m_sftp->device()->privateKeyPath()
+        << "-o" << "IdentityFile=" + KdeConnectConfig::instance()->privateKeyPath()
         << "-o" << "StrictHostKeyChecking=no" //Do not ask for confirmation because it is not a known host
-        << "-o" << "UserKnownHostsFile=/dev/null"; //Prevent storing as a known host
-    
+        << "-o" << "UserKnownHostsFile=/dev/null" //Prevent storing as a known host
+        << "-o" << "HostKeyAlgorithms=ssh-dss" //https://bugs.kde.org/show_bug.cgi?id=351725
+        << "-o" << "password_stdin"
+        ;
+
     m_proc->setProgram(program, arguments);
 
-    //To debug
-    //m_proc->setStandardOutputFile("/tmp/kdeconnect-sftp.out");
-    //m_proc->setStandardErrorFile("/tmp/kdeconnect-sftp.err");
-
-    cleanMountPoint();
-    
-    kDebug(debugArea()) << "Staring process: " << m_proc->program().join(" ");
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Starting process: " << m_proc->program().join(" ");
     m_proc->start();
+
+    //qCDebug(KDECONNECT_PLUGIN_SFTP) << "Passing password: " << np.get<QString>("password").toLatin1();
+    m_proc->write(np.get<QString>("password").toLatin1());
+    m_proc->write("\n");
+
 }
 
 void Mounter::onStarted()
 {
-    kDebug(debugArea()) << "Porcess started";
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Process started";
     m_started = true;
     Q_EMIT mounted();
-    
-    connect(m_proc.data(), SIGNAL(readyReadStandardError()), this, SLOT(readProcessOut()));
-    connect(m_proc.data(), SIGNAL(readyReadStandardOutput()), this, SLOT(readProcessOut()));
-    
-    m_lastActivity = QDateTime::currentDateTime();
-    
-    if (m_idleTimer.interval())
-    {
-        m_idleTimer.start();
-    }
+
+    //m_proc->setStandardOutputFile("/tmp/kdeconnect-sftp.out");
+    //m_proc->setStandardErrorFile("/tmp/kdeconnect-sftp.err");
+    connect(m_proc, &KProcess::readyReadStandardError, [this]() {
+        qCDebug(KDECONNECT_PLUGIN_SFTP) << "stderr: " << m_proc->readAll();
+    });
+    connect(m_proc, &KProcess::readyReadStandardOutput, [this]() {
+        qCDebug(KDECONNECT_PLUGIN_SFTP) << "stdout:" << m_proc->readAll();
+    });
 }
 
 void Mounter::onError(QProcess::ProcessError error)
 {
     if (error == QProcess::FailedToStart)
     {
-        kDebug(debugArea()) << "Porcess failed to start";
+        qCDebug(KDECONNECT_PLUGIN_SFTP) << "Process failed to start";
         m_started = false;
         Q_EMIT failed(i18n("Failed to start sshfs"));
     }
@@ -181,91 +169,44 @@ void Mounter::onFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
     if (exitStatus == QProcess::NormalExit)
     {
-        kDebug(debugArea()) << "Process finished (exit code: " << exitCode << ")";
-        
-        if (m_proc->property(idleTimeout_c).toBool())
-        {
-            Q_EMIT unmounted(true);
-        }
-        else
-        {
-            Q_EMIT unmounted(false);
-        }
+        qCDebug(KDECONNECT_PLUGIN_SFTP) << "Process finished (exit code: " << exitCode << ")";
+        Q_EMIT unmounted();
     }
     else
     {
-        kDebug(debugArea()) << "Porcess failed (exit code: " << exitCode << ")";
+        qCDebug(KDECONNECT_PLUGIN_SFTP) << "Process failed (exit code:" << exitCode << ")";
         Q_EMIT failed(i18n("Error when accessing to filesystem"));
     }
     
-    cleanMountPoint();
-    m_proc.reset();
-    m_started = false;  
+    unmount();
 }
 
 void Mounter::onMountTimeout()
 {
-    kDebug(debugArea()) << "Timeout: device not responding";
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Timeout: device not responding";
     Q_EMIT failed(i18n("Failed to mount filesystem: device not responding"));
 }
 
-void Mounter::onIdleTimeout()
-{
-    Q_ASSERT(m_proc.data());
-    
-    if (m_lastActivity.secsTo(QDateTime::currentDateTime()) >= m_idleTimer.interval() / 1000)
-    {
-        kDebug(debugArea()) << "Timeout: there is no activity on moutned filesystem";
-        m_proc->setProperty(idleTimeout_c, true);
-        unmount();
-    }
-}
-
-void Mounter::readProcessOut()
-{
-    Q_ASSERT(m_proc.data());
-    m_lastActivity = QDateTime::currentDateTime();
-    m_proc->readAll();
-}
-
 void Mounter::start()
 {
-    NetworkPackage np(PACKAGE_TYPE_SFTP);
-    np.set("startBrowsing", true);
-    np.set("start", true);
-    np.set("id", m_id);
-    np.set("idleTimeout", m_idleTimer.interval());
+    NetworkPackage np(PACKAGE_TYPE_SFTP_REQUEST, {{"startBrowsing", true}});
     m_sftp->sendPackage(np);
     
     m_connectTimer.start();
 }
 
-int Mounter::generateId()
-{
-    static int id = 0;
-    return id++;
-}
-
-void Mounter::cleanMountPoint()
-{
-    KProcess::execute(QStringList() << "fusermount" << "-u" << m_mpoint, 10000);
-}
-
 void Mounter::unmount()
 {
-    m_idleTimer.stop();
-    
+    qCDebug(KDECONNECT_PLUGIN_SFTP) << "Unmount" << m_proc;
     if (m_proc)
     {
-        cleanMountPoint();
-        if (m_proc)
-        {
-            m_proc->terminate();
-            QTimer::singleShot(5000, m_proc.data(), SLOT(kill()));
-            m_proc->waitForFinished();
-        }
+        auto toDestroy = m_proc;
+        m_proc = nullptr; //So we don't reenter this code path when onFinished gets called
+        toDestroy->kill();
+        delete toDestroy;
+        //Free mount point (won't always succeed if the path is in use)
+        KProcess::execute(QStringList() << "fusermount" << "-u" << m_mountPoint, 10000);
     }
-    
-    m_started = false;    
+    m_started = false;
 }
 
