@@ -21,13 +21,17 @@
 #ifndef TELEPHONYPLUGIN_H
 #define TELEPHONYPLUGIN_H
 
-#include <knotification.h>
+#include <QLoggingCategory>
+#include <QDBusInterface>
+
+#include <KNotification>
 
 #include <core/kdeconnectplugin.h>
 
-#include <KStatusNotifierItem>
+#define PACKAGE_TYPE_TELEPHONY_REQUEST QLatin1String("kdeconnect.telephony.request")
+#define PACKAGE_TYPE_SMS_REQUEST QLatin1String("kdeconnect.sms.request")
 
-#define PACKAGE_TYPE_TELEPHONY QLatin1String("kdeconnect.telephony")
+Q_DECLARE_LOGGING_CATEGORY(KDECONNECT_PLUGIN_TELEPHONY)
 
 class TelephonyPlugin
     : public KdeConnectPlugin
@@ -37,13 +41,20 @@ class TelephonyPlugin
 public:
     explicit TelephonyPlugin(QObject *parent, const QVariantList &args);
 
+    bool receivePackage(const NetworkPackage& np) override;
+    void connected() override { }
+
 public Q_SLOTS:
-    virtual bool receivePackage(const NetworkPackage& np);
-    virtual void connected() { }
+    void sendMutePackage();
+
+private Q_SLOTS:
+    void sendSms(const QString& phoneNumber, const QString& messageBody);
+    void showSendSmsDialog();
 
 private:
     KNotification* createNotification(const NetworkPackage& np);
 
+    QDBusInterface m_telepathyInterface;
 };
 
 #endif
