@@ -17,14 +17,13 @@
  *************************************************************************************/
 
 #include "../core/backends/lan/socketlinereader.h"
+#include "../core/backends/lan/server.h"
 
 #include <QTest>
-#include <QTcpServer>
-#include <QTcpSocket>
+#include <QSslSocket>
 #include <QProcess>
 #include <QEventLoop>
 #include <QTimer>
-#include <unistd.h>
 
 class TestSocketLineReader : public QObject
 {
@@ -40,21 +39,22 @@ private:
     QTimer mTimer;
     QEventLoop mLoop;
     QList<QByteArray> mPackages;
-    QTcpServer *mServer;
-    QTcpSocket *mConn;
+    Server *mServer;
+    QSslSocket *mConn;
     SocketLineReader *mReader;
 };
 
 void TestSocketLineReader::initTestCase()
 {
-    mServer = new QTcpServer(this);
+    mServer = new Server(this);
+
     QVERIFY2(mServer->listen(QHostAddress::LocalHost, 8694), "Failed to create local tcp server");
 
     mTimer.setInterval(4000);//For second is more enough to send some data via local socket
     mTimer.setSingleShot(true);
     connect(&mTimer, SIGNAL(timeout()), &mLoop, SLOT(quit()));
 
-    mConn = new QTcpSocket(this);
+    mConn = new QSslSocket(this);
     mConn->connectToHost(QHostAddress::LocalHost, 8694);
     connect(mConn, SIGNAL(connected()), &mLoop, SLOT(quit()));
     mTimer.start();
@@ -78,9 +78,9 @@ void TestSocketLineReader::socketLineReader()
         QTest::qSleep(1000);
     }
 
-    QTcpSocket *sock = mServer->nextPendingConnection();
+    QSslSocket *sock = mServer->nextPendingConnection();
 
-    QVERIFY2(sock != 0, "Could not open a connection to the client");
+    QVERIFY2(sock != nullptr, "Could not open a connection to the client");
 
     mReader = new SocketLineReader(sock, this);
     connect(mReader, SIGNAL(readyRead()), SLOT(newPackage()));
@@ -117,6 +117,6 @@ void TestSocketLineReader::newPackage()
 }
 
 
-QTEST_MAIN(TestSocketLineReader)
+QTEST_GUILESS_MAIN(TestSocketLineReader)
 
 #include "testsocketlinereader.moc"
